unit cDataStructs;

interface

uses
  Classes,       // TList
  SysUtils,      // Exception

  // Delphi Fundamentals (L0)
  cStream;       // TStream



{                                                                              }
{ DataStructs v0.02 (L0)                                                       }
{                                                                              }
{   A collection of basic data structures with implementations for primary     }
{     memory and secondary memory (streams).                                   }
{                                                                              }
{                                                                              }
{ This unit is copyright  2000 by David Butler (david@e.co.za)                }
{                                                                              }
{ This unit is part of Delphi Fundamentals, it's original name is              }
{ cDataStructs.                                                                }
{                                                                              }
{ Revision history:                                                            }
{   2000/02/08  v0.01  Initial version. AArray, TArray and TStreamArray.       }
{   2000/04/01  v0.02  Moved ASet from cMaths.                                 }
{                      Added Find methods to ASet.                             }
{                      Changed TFlatSet implementation to store to AArray      }
{                      instead of in memory only (TBits).                      }
{                                                                              }
{                                                                              }



{                                                                              }
{ AArray                                                                       }
{   Abstract base class for an array of Integers.                              }
{   TArray stores the values in memory using a dynamic array.                  }
{   TStreamArray stores the values in a stream using the ATRecordStream        }
{     interface. It can be used to access large arrays from secondary memory.  }
{                                                                              }
type
  AArray = class
    protected
    Procedure SetItem (const Idx : Integer; const Value : Integer); virtual; abstract;
    Function  GetItem (const Idx : Integer) : Integer; virtual; abstract;
    Function  GetCount : Integer; virtual; abstract;
    Procedure SetCount (const NewCount : Integer); virtual; abstract;

    public
    Property  Item [const Idx : Integer] : Integer read GetItem write SetItem; default;
    Property  Count : Integer read GetCount write SetCount;

    Procedure Fill (const Value : Integer = 0); virtual; abstract;
    Procedure FillRange (const StartValue : Integer = 0; const Increment : Integer = 1); virtual; abstract;
  end;

  TArray = class (AArray)
    protected
    FData : Array of Integer;

    Procedure SetItem (const Idx : Integer; const Value : Integer); override;
    Function  GetItem (const Idx : Integer) : Integer; override;

    Function  GetCount : Integer; override;
    Procedure SetCount (const NewCount : Integer); override;

    public
    Procedure Fill (const Value : Integer); override;
    Procedure FillRange (const StartValue : Integer; const Increment : Integer); override;
  end;

  TStreamArray = class (AArray)
    protected
    FRec : TFixedLenRecordStreamer;

    Procedure SetItem (const Idx : Integer; const Value : Integer); override;
    Function  GetItem (const Idx : Integer) : Integer; override;

    Function  GetCount : Integer; override;
    Procedure SetCount (const NewCount : Integer); override;

    public
    Constructor Create (const S : TStream);
    Destructor Destroy; override;

    Procedure Fill (const Value : Integer); override;
    Procedure FillRange (const StartValue : Integer; const Increment : Integer); override;
  end;



{                                                                              }
{ Sets                                                                         }
{                                                                              }

{                                                                              }
{ ASet                                                                         }
{   Abstract base class for set (array of bit) implementations.                }
{   Bits are defined as False at initialization.                               }
{   Find finds Count consequetive bits set to Value, starting at (and          }
{     including) Start. It returns the index of the leftmost bit or -1 if      }
{     not found.                                                               }
{                                                                              }
type
  ASet = class
    protected
    Function  GetBit (const Idx : Integer) : Boolean; virtual; abstract;
    Procedure SetBit (const Idx : Integer; const Value : Boolean); virtual; abstract;

    public
    Procedure Clear; virtual; abstract;
    Procedure Invert; virtual; abstract;
    Function  GetRange (const Low, High : Integer; const Value : Boolean) : Boolean; virtual; abstract;
    Procedure SetRange (const Low, High : Integer; const Value : Boolean); virtual; abstract;
    Property  Bit [const Idx : Integer] : Boolean read GetBit write SetBit; default;
    Function  Find (const Value : Boolean = False; const Start : Integer = 0;   // Find bit
              const FindForward : Boolean = True) : Integer; overload; virtual; abstract;
    Function  Find (const Value : Boolean = False; const Start : Integer = 0;   // Find range
              const Count : Integer = 1; const FindForward : Boolean = True) : Integer; overload; virtual; abstract;
  end;



{                                                                              }
{ TFlatSet                                                                     }
{   A flat set (stored in AArray)                                              }
{   Pros: Fast allocation/deallocation.                                        }
{         1 bit of memory for each entry.                                      }
{   Cons: Bad memory usage if bits are randomly distributed                    }
{                                                                              }
type
  TFlatSet = class (ASet)
    protected
    FStore : AArray;

    Function  GetBit (const Idx : Integer) : Boolean; override;
    Procedure SetBit (const Idx : Integer; const Value : Boolean); override;

    public
    Constructor Create (const Store : AArray);
    Destructor Destroy; override;

    Procedure Clear; override;
    Procedure SetRange (const Low, High : Integer; const Value : Boolean); override;
    Function  GetRange (const Low, High : Integer; const Value : Boolean) : Boolean; override;
    Function  Find (const Value : Boolean = False; const Start : Integer = 0;   // Find bit
              const FindForward : Boolean = True) : Integer; override;
  end;



{                                                                              }
{ TRange                                                                       }
{   Store Low..High value.                                                     }
{   HasElement returns True if I in Low..High.                                 }
{   Overlap returns True if overlapping ranges.                                }
{   Touch returns True if touching (adjacent) range.                           }
{   Inside returns True if inside (contained) range.                           }
{   Merge merges adjacent/overlapping ranges. It raises an exception if the    }
{     ranges are not adjacent/overlapping.                                     }
{                                                                              }
type
  ERange = class (Exception);
  TRange = class
    protected
    FLow, FHigh : Integer;

    public
    Constructor Create (const Low, High : Integer);

    Function  HasElement (const I : Integer) : Boolean;
    Function  Overlap (const Low, High : Integer) : Boolean;
    Function  Touch (const Low, High : Integer) : Boolean;
    Function  Inside (const Low, High : Integer) : Boolean;
    Procedure Merge (const Low, High : Integer);
  end;



{                                                                              }
{ TRangeSet                                                                    }
{   A set (stored as a list of ranges)                                         }
{   Pros: Good memory usage if set bits occur sparsely in ranges               }
{   Cons: Bad memory usage if bits are randomly distributed                    }
{                                                                              }
type
  TRangeSet = class (ASet)
    FRanges : TList;

    Constructor Create;
    Destructor Destroy; override;
    Procedure Clear; override;
    Procedure Invert; override;
    Procedure SetRange (const Low, High : Integer; const Value : Boolean); override;
    Function  GetBit (const Idx : Integer) : Boolean; override;
    Function  GetRange (const Low, High : Integer; const Value : Boolean) : Boolean; override;
    Procedure SetBit (const Idx : Integer; const Value : Boolean); override;
  end;



{                                                                              }
{ TSparseFlatSet                                                               }
{   A flat set (stored as a array of pointers to Delphi sets of Byte)          }
{   Pros: Good performance for randomly distributed bits, especially if they   }
{   appear in little clumps.                                                   }
{   Cons: Bad performance for ranges.                                          }
{                                                                              }
type
  TPointerArray = Array [0..MaxLongInt div 256] of Pointer;
  PPointerArray = ^TPointerArray;
  TDelphiSet = Set of Byte; // 256 elements
  PDelphiSet = ^TDelphiSet;

const
  EmptyDelphiSet : TDelphiSet = [];
  CompleteDelphiSet : TDelphiSet = [0..255];

type
  TSparseFlatSet = class (ASet)
    FSetList        : PPointerArray;
    FSetListEntries : Integer;

    Destructor Destroy; override;
    Procedure Clear; override;
    Procedure Invert; override;
    Function  GetBit (const Idx : Integer) : Boolean; override;
    Procedure SetBit (const Idx : Integer; const Value : Boolean); override;
    Procedure SetRange (const Low, High : Integer; const Value : Boolean); override;
  end;



implementation

uses
  Math;



{                                                                              }
{ AArray                                                                       }
{                                                                              }
Procedure TArray.SetItem (const Idx : Integer; const Value : Integer);
  Begin
    FData [Idx] := Value;
  End;

Function TArray.GetItem (const Idx : Integer) : Integer;
  Begin
    Result := FData [Idx];
  End;

Function TArray.GetCount : Integer;
  Begin
    Result := Length (FData);
  End;

Procedure TArray.SetCount (const NewCount : Integer);
  Begin
    SetLength (FData, NewCount);
  End;

Procedure TArray.Fill (const Value : Integer);
var I : Integer;
  Begin
    For I := 0 to Length (FData) - 1 do
      FData [I] := Value;
  End;

Procedure TArray.FillRange (const StartValue : Integer; const Increment : Integer);
var I, J : Integer;
  Begin
    J := StartValue;
    For I := 0 to Length (FData) - 1 do
      begin
        FData [I] := J;
        Inc (J, Increment);
      end;
  End;

Constructor TStreamArray.Create (const S : TStream);
  Begin
    inherited Create;
    FRec := TFixedLenRecordStreamer.Create (S, Sizeof (Integer));
  End;

Destructor TStreamArray.Destroy;
  Begin
    FreeAndNil (FRec);
    inherited Destroy;
  End;

Procedure TStreamArray.SetItem (const Idx : Integer; const Value : Integer);
  Begin
    FRec.Write (Idx, 1, Value);
  End;

Function TStreamArray.GetItem (const Idx : Integer) : Integer;
  Begin
    FRec.Read (Idx, 1, Result);
  End;

Function TStreamArray.GetCount : Integer;
  Begin
    Result := FRec.RecordCount;
  End;

Procedure TStreamArray.SetCount (const NewCount : Integer);
  Begin
    FRec.RecordCount := NewCount;
  End;

Procedure TStreamArray.Fill (const Value : Integer);
const BlockSize = 1024;
var I, C : Integer;
    A    : TArray;
  Begin
    A := TArray.Create;
    C := Min (FRec.RecordCount, BlockSize);
    A.Count := C;
    A.Fill (Value);
    For I := 0 to (FRec.RecordCount - 1) div C do
      FRec.Write (I * C, Min (C, FRec.RecordCount - I * C), A.FData);
  End;

Procedure TStreamArray.FillRange (const StartValue : Integer; const Increment : Integer);
const BlockSize = 1024;
var I, C : Integer;
    A    : TArray;
  Begin
    A := TArray.Create;
    C := Min (FRec.RecordCount, BlockSize);
    A.Count := C;
    For I := 0 to (FRec.RecordCount - 1) div C do
      begin
        A.FillRange (StartValue + I * C * Increment, Increment);
        FRec.Write (I * C, Min (C, FRec.RecordCount - I * C), A.FData [0]);
      end;
  End;



{                                                                              }
{ TFlatSet                                                                     }
{                                                                              }
const
  IntBytes = Sizeof (Integer);
  IntBits  = IntBytes * 8;
  TrueInt  : Integer = $FFFFFFFF;
  FalseInt : Integer = $00000000;

Constructor TFlatSet.Create (const Store : AArray);
  Begin
    inherited Create;
    FStore := Store;
  End;

Destructor TFlatSet.Destroy;
  Begin
    FStore.Free;
    FStore := nil;
    inherited Destroy;
  End;

Procedure TFlatSet.Clear;
  Begin
    FStore.Count := 0;
  End;

Function TFlatSet.GetBit (const Idx : Integer) : Boolean;
  Begin
    Result := FStore [Idx div IntBits] shr (Idx mod IntBits) and 1 = 1;
  End;

Procedure TFlatSet.SetBit (const Idx : Integer; const Value : Boolean);
var I, B : Integer;
  Begin
    I := Idx div IntBits;
    if I >= FStore.Count then
      FStore.Count := I;
    B := 1 shl (Idx mod IntBits);
    if Value then
      FStore [I] := FStore [I] or B else
      FStore [I] := FStore [I] and not B;
  End;

// Returns Integer with bits L to H set
Function BitRange (const L, H : Byte) : Integer; Assembler;
  Asm
    push ecx
    mov cl, L
    mov eax, TrueInt
    or cl, cl
    jz @CheckHigh
    shr eax, cl
    shl eax, cl
  @CheckHigh:
    mov cl, IntBits - 1
    sub cl, H
    or cl, cl
    jz @Fin
    shl eax, cl
    shr eax, cl
  @Fin:
    pop ecx
  End;

Function TFlatSet.GetRange (const Low, High : Integer; const Value : Boolean) : Boolean;
var B, I, IL, IH : Integer;
  Begin
    // Check bits in FStore [IL]
    IL := Low div IntBits;
    IH := High div IntBits;
    B := BitRange (Low mod IntBits, IntBits - 1);
    I := FStore [IL];
    if Value then
      Result := I or B = I else
      Result := I and not B = I;
    if not Result or (IL = IH) then
      exit;

    // Check bits in FStore [IH]
    B := BitRange (0, High mod IntBits);
    I := FStore [IH];
    if Value then
      Result := I or B = I else
      Result := I and not B = I;
    if not Result or (IH = IL + 1) then
      exit;

    // Check bits in FStore [IL + 1..IR - 1]
    For I := IL + 1 to IH - 1 do
      if (Value and (FStore [I] <> TrueInt)) or
         (not Value and (FStore [I] <> FalseInt)) then
        begin
          Result := False;
          exit;
        end;

    Result := True;
  End;

Procedure TFlatSet.SetRange (const Low, High : Integer; const Value : Boolean);
var B, I, IL, IH : Integer;
  Begin
    IL := Low div IntBits;
    IH := High div IntBits;
    FStore.Count := Max (FStore.Count, IH + 1);

    // Set bits in FStore [IL]
    if IH = IL then
      B := BitRange (Low mod IntBits, High mod IntBits) else
      B := BitRange (Low mod IntBits, IntBits - 1);
    I := FStore [IL];
    if Value then
      FStore [IL] := I or B else
      FStore [IL] := I and not B;
    if IH = IL then
      exit;

    // Set bits in FStore [IH]
    B := BitRange (0, High mod IntBits);
    I := FStore [IH];
    if Value then
      FStore [IH] := I or B else
      FStore [IH] := I and not B;
    if IH = IL + 1 then
      exit;

    // Set bits in FStore [IL + 1..IR - 1]
    For I := IL + 1 to IH - 1 do
      if Value then
        FStore [I] := TrueInt else
        FStore [I] := FalseInt;
  End;

Function TFlatSet.Find (const Value : Boolean = False; const Start : Integer = 0; const FindForward : Boolean = True) : Integer;
var I : Integer;
  Begin
    I := Start;
    While (I >= 0) and (I < FStore.Count * IntBits) do
      begin
        if not (GetBit (I) xor Value) then
          begin
            Result := I;
            exit;
          end;
        if FindForward then
          Inc (I) else
          Dec (I);
      end;
    Result := -1;
  End;

// Function FlatSet.Find (const Value : Boolean = False; const Start : Integer = 0; const Count : Integer = 1; const FindForward : Boolean = True) : Integer;


// Invert


{                                                                              }
{ TRange                                                                       }
{                                                                              }
Constructor TRange.Create (const Low, High : Integer);
  Begin
    inherited Create;
    FLow := Low;
    FHigh := High;
  End;

Function TRange.Overlap (const Low, High : Integer) : Boolean;
  Begin
    Result := (High >= FLow) xor (Low <= FHigh);
  End;

Function TRange.Touch (const Low, High : Integer) : Boolean;
  Begin
    Result := (High >= FLow - 1) xor (Low <= FHigh + 1);
  End;

Function TRange.Inside (const Low, High : Integer) : Boolean;
  Begin
    Result := (FLow >= Low) and (FHigh <= High);
  End;

Function TRange.HasElement (const I : Integer) : Boolean;
  Begin
    Result := (I >= FLow) and (I <= FHigh);
  End;

Procedure TRange.Merge (const Low, High : Integer);
  Begin
    if (High >= FLow - 1) xor (Low <= FHigh + 1) then // Touch
      begin
        if Low < FLow then
          FLow := Low;
        if High > FHigh then
          FHigh := High;
      end else
      raise ERange.Create ('Can not merge range');
  End;





{                                                                              }
{ TRangeSet                                                                    }
{                                                                              }
Constructor TRangeSet.Create;
  Begin
    inherited Create;
    FRanges := TList.Create;
  End;

Destructor TRangeSet.Destroy;
  Begin
    Clear;
    FRanges.Free;
    FRanges := nil;
    inherited Destroy;
  End;

Procedure TRangeSet.Clear;
var I : Integer;
  Begin
    For I := 0 to FRanges.Count - 1 do
      TRange (FRanges [I]).Free;
    FRanges.Clear;
  End;

Procedure TRangeSet.Invert;
var I      : Integer;
    R, Pre : TRange;
  Begin
    if (FRanges.Count > 0) and (TRange (FRanges [0]).FLow > 0) then
      Pre := TRange.Create (0, TRange (FRanges [0]).FLow - 1) else
      Pre := nil;
    For I := 0 to FRanges.Count - 2 do
      begin
        R := TRange (FRanges [I]);
        R.FLow := R.FHigh + 1;
        R.FHigh := TRange (FRanges [I + 1]).FLow - 1;
      end;
    if FRanges.Count > 0 then
      if TRange (FRanges [FRanges.Count - 1]).FHigh = MaxInt then
      begin
        TRange (FRanges [FRanges.Count - 1]).Free;
        FRanges.Delete (FRanges.Count - 1);
      end else
      begin
        TRange (FRanges [FRanges.Count - 1]).FLow := TRange (FRanges [FRanges.Count - 1]).FHigh + 1;
        TRange (FRanges [FRanges.Count - 1]).FHigh := MaxInt;
      end;
    if Assigned (Pre) then
      FRanges.Insert (0, Pre);
  End;

Procedure TRangeSet.SetRange (const Low, High : Integer; const Value : Boolean);
var I, J, K : Integer;
    R : TRange;
  Begin
    I := 0;
    K := FRanges.Count;
    While (I <= K - 1) and (Low > TRange (FRanges [I]).FHigh) do
      Inc (I);
    if (I = K) or  // append
       ((Low < TRange (FRanges [I]).FLow) and (High < TRange (FRanges [I]).FLow)) then // insert
      begin
        if Value then
          FRanges.Insert (I, TRange.Create (Low, High));
        exit;
      end;
    // I = first block that overlaps

    J := I;
    While (J < K - 1) and (TRange (FRanges [J + 1]).FLow <= High) do
      Inc (J);
    // J = last block that overlaps

    if not Value then // clear range (NOT IMPL)
      begin
        if I = J then
          begin
            R := FRanges [I];
            if R.Inside (Low, High) then
              begin
                R.Free;
                FRanges.Delete (I);
                exit;
              end;
          end else
          begin
            For K := I + 1 to J - 1 do
              begin
                TRange (FRanges [I + 1]).Free;
                FRanges.Delete (I + 1);
              end;
          end;
      end else // set range
      begin
        R := FRanges [I];
        if I = J then
          begin
            if Low < R.FLow then
              R.FLow := Low;
            if High > R.FHigh then
              R.FHigh := High;
          end else
          begin
            R.FLow := Low;
            R.FHigh := High;
            For K := I + 1 to J do
              begin
                TRange (FRanges [I + 1]).Free;
                FRanges.Delete (I + 1);
              end;
          end;
      end;
  End;

Function TRangeSet.GetBit (const Idx : Integer) : Boolean;
var I, J : Integer;
  Begin
    I := 0;
    J := FRanges.Count - 1;
    While (I <= J) and (Idx > TRange (FRanges [I]).FHigh) do
      Inc (I);
    Result := (I <= J) and (Idx >= TRange (FRanges [I]).FLow);
  End;

Function TRangeSet.GetRange (const Low, High : Integer; const Value : Boolean) : Boolean;
var I, J : Integer;
  Begin
    I := 0;
    J := FRanges.Count - 1;
    While (I <= J) and (Low > TRange (FRanges [I]).FHigh) do
      Inc (I);
    Result := (I <= J) and (Low >= TRange (FRanges [I]).FLow) and (High <= TRange (FRanges [I]).FHigh);
  End;

Procedure TRangeSet.SetBit (const Idx : Integer; const Value : Boolean);
  Begin
    SetRange (Idx, Idx, Value);
  End;



{                                                                              }
{ TSparseFlatSet                                                               }
{                                                                              }
Destructor TSparseFlatSet.Destroy;
  Begin
    Clear;
    inherited Destroy;
  End;

Procedure TSparseFlatSet.Clear;
var F : Integer;
  Begin
    if Assigned (FSetList) then
      begin
        For F := 0 to FSetListEntries - 1 do
          if Assigned (FSetList^ [F]) then
            Release (PDelphiSet (FSetList^ [F]));
        FreeMem (FSetList, FSetListEntries * Sizeof (Pointer));
        FSetList := nil;
        FSetListEntries := 0;
      end;
  End;

Procedure TSparseFlatSet.Invert;
var F : Integer;
  Begin
    For F := 0 to FSetListEntries - 1 do
      if Assigned (FSetList^ [F]) then
        PDelphiSet (FSetList^ [F])^ := CompleteDelphiSet - PDelphiSet (FSetList^ [F])^;
  End;

Function TSparseFlatSet.GetBit (const Idx : Integer) : Boolean;
var SetIdx : Integer;
  Begin
    SetIdx := Idx shr 8;
    Result := (SetIdx < FSetListEntries) and
              Assigned (FSetList^ [SetIdx]) and
              (Byte (Idx and $FF) in PDelphiSet (FSetList^ [SetIdx])^);
  End;

Procedure TSparseFlatSet.SetBit (const Idx : Integer; const Value : Boolean);
var I,
    SetIdx : Integer;
    S      : PDelphiSet;
  Begin
    SetIdx := Idx shr 8;
    if SetIdx >= FSetListEntries then
      if Value then
        begin
          I := FSetListEntries;
          FSetListEntries := SetIdx + 1;
          ReallocMem (FSetList, FSetListEntries * Sizeof (Pointer));
          FillChar (FSetList^ [I], (FSetListEntries - I) * Sizeof (Pointer), #0);
        end else
        exit;
    S := FSetList^ [SetIdx];
    if not Assigned (S) then
      if Value then
        begin
          New (S);
          S^ := [];
          FSetList^ [SetIdx] := S;
        end else
        exit;
    Include (S^, Byte (Idx and $FF));
  End;

Procedure TSparseFlatSet.SetRange (const Low, High : Integer; const Value : Boolean);

  Procedure SetValue (const S : TDelphiSet; const SetIdx : Integer);
  var D : PDelphiSet;
    Begin
      D := FSetList^ [SetIdx];
      if not Assigned (D) then
        begin
          if Value then
            begin
              New (D);
              D^ := S;
              FSetList^ [SetIdx] := D;
            end;
        end else
      if Value then
        D^ := D^ + S else
        D^ := D^ - S;
    End;

var I, LowSet,
    HighSet : Integer;

  Begin
    LowSet := Low shr 8;
    HighSet := High shr 8;
    if HighSet >= FSetListEntries then
      begin
        I := FSetListEntries;
        FSetListEntries := HighSet + 1;
        ReallocMem (FSetList, FSetListEntries * Sizeof (Pointer));
        FillChar (FSetList^ [I], (FSetListEntries - I) * Sizeof (Pointer), #0);
      end;
    if LowSet = HighSet then
      SetValue ([Byte (Low and $FF)..Byte (High and $FF)], LowSet) else
      begin
        SetValue ([Byte (Low and $FF)..$FF], LowSet);
        SetValue ([0..Byte (High and $FF)], HighSet);
        For I := LowSet + 1 to HighSet - 1 do
          SetValue (CompleteDelphiSet, I);
      end;
  End;

end.

